!C                                               STR77.FOR
!C
!CSTRING HANDLING SUBROUTINE LIBRARY STR77
!C****************************************
!C
!CCONVERTED TO FORTRAN 77 (ANSI X3.9-1978) BY L.P. EKSTROM, LUND.
!CMOST ROUTINES ARE SLIGHT MODIFICATIONS OF THE ORNL SUBR. LIBRARY
!CFORSTR AND NSDLIB (SINGLE AND DOUBLE PRECISION). THE MAIN
!CMODIFICATIONS ARE:
!C
!C   1) STRICT FORTRAN 77 STANDARD IS FOLLOWED.
!C
!C   2) ALL STRINGS ARE STORED AS CHARACTER STRINGS.
!C
!C   3) SOME ROUTINES HAVE BEEN RENAMED, AND SOME ROUTINES ARE
!C     NOT NEEDED SINCE INTRINSIC FORTRAN 77 ROUTINES CAN BE USED.
!C
!CApril 1994: Removed all occurrencies of str=str(n:) with
!CDUMSTR*80  /lpe
!C
      SUBROUTINE SRESET(STR,N)
!C
!CPROGRAM UNIT 1
!CNULLS STRING SO THAT ALL BUT N FIRST CHARACTERS ARE NULL
!CDUMMY ARGUMENTS:
!CSTR   THE STRING (ASSIGNED)
!CN     WANTED LENGTH
!C
      CHARACTER*(*) STR

      LM=LEN(STR)
      IF(N.LT.LM) THEN
         DO 10 I=N+1,LM
            STR(I:I)=CHAR(0)
10          CONTINUE
      ENDIF
      RETURN
      END
      SUBROUTINE SRESTO(STR)
!C
!CPROGRAM UNIT 2
!CRESTORES PARTLY NULLED STRING, SO THAT IT IS PADDED WITH
!CBLANKS INSTEAD OF NULLS.
!CDUMMY ARGUMENT:
!CSTR   THE STRING (ASSIGNED)
!C
      CHARACTER*(*) STR

      LM=LEN(STR)
      LC=LENSTR(STR)
      IF(LC.LT.LM) STR(LC+1:)='                                      '
      RETURN
      END
      INTEGER FUNCTION LENSTR(STR)
!C
!CPROGRAM UNIT 3
!CFINDS CURRENT LENGTH (I.E. NUMBER OF CHARACTERS BEFORE A
!CNULL CHARACTER) OF STRING STR
!CDUMMY ARGUMENT:
!CSTR   THE STRING
!C
      CHARACTER*(*) STR

      LENSTR=LEN(STR)
      M=INDEX(STR,CHAR(0))
      IF(M.GT.0) LENSTR=M-1
      RETURN
      END
      SUBROUTINE UNSTR(STR,N,PACK)
!C
!CPROGRAM UNIT 4
!CPACKS N FIRST CHARACTERS OF STR INTO PACK WITH NBYTE BYTES/WORD.
!C**** NOTE: NO CHECKING ON THE DIMENSION OF THE INTEGER ARRAY
!CPACK IS POSSIBLE, SO IT IS UP TO THE USER TO SEE THAT IT IS
!CBIG ENOUGH.
!CDUMMY ARGUMENTS:
!CSTR   THE STRING
!CN     NUMBER OF CHARACTERS TO BE PACKED
!CPACK  INTEGER ARRAY WHERE CHARACTERS ARE PACKED (ASSIGNED)
!C
!CNBYTE IS NUMBER OF BYTES IN AN INTEGER (COMPUTER DEPENDENT!C***)
!C**** NOTE THAT THE ORDER OF THE CHARACTERS IN A WORD IS COMPUTER
!CDEPENDENT ****
      PARAMETER (NBYTE=4)
      CHARACTER STR*(*)
      INTEGER PACK(*)

      IF(N.GT.LEN(STR).OR.N.LE.0) GO TO 90
         DO 10 I=1,N,NBYTE
            IWORD=0
!C      FIRST CHARACTER IN MOST SIGNIFICANT BYTE
!C       DO 5 J=1,NBYTE
!C      FIRST CHARACTER IN LEAST SIGNIFICANT BYTE
            DO 5 J=NBYTE,1,-1
               JCH=I+J-1
               IF(JCH.GT.N) GO TO 7
               IWORD=IWORD*256 + ICHAR(STR(JCH:JCH))
5              CONTINUE
7        PACK(I/NBYTE+1)=IWORD
10       CONTINUE
90    RETURN
      END
      SUBROUTINE ALIGN(STR,N)
!C
!CPROGRAM UNIT 5
!CALIGNS DECIMAL POINT OR FIRST SPACE AFTER FIRST NUMBER
!CTO POSITION N. IF THAT IS IMPOSSIBLE, STR IS LEFT UNTOUCHED.
!CDUMMY ARGUMENTS:
!CSTR   THE STRING CONTAINING A NUMBER (ASSIGNED)
!CN     THE WANTED POSITION OF DECIMAL POINT
!C
      CHARACTER STR*(*),WORK*30

      LC=LENSTR(STR)
      WORK=STR
      CALL LBSUP(WORK)
      CALL TBSUP(WORK)
      IAL=INDEX(WORK,'.')
      IF(IAL.LE.0) IAL=INDEX(WORK,' ')
      IF(IAL.LE.0) IAL=LENSTR(WORK)+1
      IF(IAL.LE.0.OR.IAL.GT.N) GO TO 90
      IF(IAL.EQ.N) GO TO 50
      IBL=N-IAL
      DO 10 I=1,IBL
10       CALL PUTIN(WORK,1,' ',1)
      IF(LENSTR(WORK).GT.LC) GO TO 90
50    CALL SRESTO(WORK)
      CALL SRESET(WORK,LC)
      STR=WORK
90    RETURN
      END
      SUBROUTINE SQUEEZ(STR,STR1)
!C
!CPROGRAM UNIT 6
!CREMOVES ALL OCCURRANCES OF STR1 FROM STR.
!CSTR1 MUST BE OF THE PROPER LENGTH, I.E. CONTANING
!CNO NULLS.
!CDUMMY ARGUMENTS:
!CSTR   THE STRING TO BE SQUEEZED (ASSIGNED)
!CSTR1  THE STRING TO BE REMOVED FROM STR
!C
      CHARACTER*(*) STR,STR1,DUMSTR*80

      LC=LENSTR(STR)
      LC1=LENSTR(STR1)
10    CONTINUE
      IPOS=INDEX(STR,STR1)
      IF(IPOS.LE.0.OR.IPOS.GT.LC-LC1+1) GO TO 80
      DUMSTR=STR
      STR(IPOS:LC-LC1)=DUMSTR(IPOS+LC1:LC)
      LC=L!C-LC1
      GO TO 10
80    CALL SRESET(STR,LC)
      RETURN
      END
      SUBROUTINE LBSUP(STR)
!C
!CPROGRAM UNIT 7
!CTAKES AWAY LEADING/TRAILING BLANKS.
!CDUMMY ARGUMENT:
!CSTR   THE STRING (ASSIGNED)
!C
      CHARACTER STR*(*),DUMSTR*80

      LC=LENSTR(STR)
      DO 10 I=1,L!C
         IF(STR(1:1).NE.' ') GO TO 90
         DUMSTR=STR(2:)//CHAR(0)
         STR=DUMSTR
10       CONTINUE
90    RETURN

      ENTRY TBSUP(STR)
      LC=LENSTR(STR)
      DO 110 I=LC,1,-1
         IF(STR(I:I).NE.' ') GO TO 190
         STR(I:I)=CHAR(0)
110      CONTINUE
190   RETURN
      END
      SUBROUTINE SUPALF(STR)
!C
!CPROGRAM UNIT 8
!CWILL CONVERT ALL ALPHA CHARACTERS (EXCEPT -, +, . AND E) IN STR
!CINTO BLANKS.
!CFOR USE IN NUMERIC CONVERSION ROUTINES.
!CDUMMY ARGUMENT:
!CSTR   THE STRING (ASSIGNED)
!C
      CHARACTER*(*) STR
      CHARACTER*14 KEEP
      DATA KEEP /'-+.E0123456789'/

      L=LENSTR(STR)
      IF(L.LE.0) GO TO 90
      DO 10 I=1,L
         DO 5 J=1,14
            IF(STR(I:I).EQ.KEEP(J:J)) GO TO 10
5           CONTINUE
         STR(I:I)=' '
10       CONTINUE
90    RETURN
      END
      SUBROUTINE PADRT(STR,L)
!C
!CPROGRAM UNIT 9
!CMAKE STR L CHARACTERS LONG BY EITHER TAKING AWAY BLANKS OR
!CFILLING WITH BLANKS TO THE RIGHT/LEFT.
!CDUMMY ARGUMENTS:
!CSTR   THE STRING (ASSIGNED)
!CL     WANTED CURRENT LENGTH OF STR
!C
      CHARACTER*(*) STR,DUMSTR*80

      CALL TBSUP(STR)
      LC=LENSTR(STR)
      IF(LC.GE.L) RETURN
      CALL SRESTO(STR)
      CALL SRESET(STR,L)
      RETURN

      ENTRY PADLFT(STR,L)
      IF(L.GT.LEN(STR)) RETURN
      CALL LBSUP(STR)
      LC=LENSTR(STR)
      IF(LC.GE.L) RETURN
      DUMSTR=STR
      STR(L-LC+1:L)=DUMSTR(1:L)
      STR(1:L-LC)='                                                  '
      RETURN
      END
      SUBROUTINE ZERFIL(STR,L)
!C
!CPROGRAM UNIT 10
!CMAKE STR L CHARACTERS LONG BY FILLING WITH ZEROS TO THE LEFT
!CDUMMY ARGUMENTS:
!CSTR   THE STRING (ASSIGNED)
!CL     WANTED CURRENT LENGTH OF STR
!C
      CHARACTER*(*) STR,DUMSTR*80

      IF(L.GT.LEN(STR)) RETURN
      CALL LBSUP(STR)
      LC=LENSTR(STR)
      IF(LC.LT.L) THEN
         DUMSTR=STR
         STR(L-LC+1:L)=DUMSTR(1:LC)
         STR(1:L-LC)='0000000000000000000000000000000000000000000000'
      ENDIF
      RETURN
      END
      SUBROUTINE PUTIN(OBJ,LOC,SRC,LD)
!C
!CPROGRAM UNIT 11
!CINSERTS (BY PUSHING TO THE RIGHT) THE FIRST LD CHARACTERS
!COF SRC INTO OBJ STARTING AT LOC.
!CDUMMY ARGUMENTS:
!COBJ   OBJECT STRING (ASSIGNED)
!CLOC   STARTING POSITION
!CSRC   SOURCE STRING
!CLD    NUMBER OF WANTED CHARACTERS FROM SRC
!C
      CHARACTER*(*) OBJ,SRC
      CHARACTER*80 DUMSTR

      IF(LOC.GT.LENSTR(OBJ)+1) RETURN
      IF(LD.GT.LEN(SRC)) RETURN
      IF((LOC+LD-1).GT.LEN(OBJ)) RETURN
      IF(LOC.GT.0) THEN
         DUMSTR=OBJ
         DUMSTR(LOC+LD:)=OBJ(LOC:)
         DUMSTR(LOC:LOC+LD-1)=SRC(1:LD)
         OBJ=DUMSTR
      ENDIF
      RETURN
      END
      SUBROUTINE APPEND(OBJ,SRC)
!C
!CPROGRAM UNIT 12
!CAPPEND SRC TO OBJ
!CDUMMY ARGUMENTS:
!COBJ   OBJECT STRING (ASSIGNED)
!CSRC   SOURCE STRING
!C
      CHARACTER*(*) OBJ,SRC

      CALL PUTIN(OBJ,LENSTR(OBJ)+1,SRC,LENSTR(SRC))
      RETURN
      END
      INTEGER FUNCTION ITYPES(STR)
!C
!CPROGRAM UNIT 13
!CRETURNS THE TYPE CODE OF THE STRING STR.
!C   NULL                    0
!C   NUMERIC (0-9)           1
!C   ALPHA (A-Z OR a-z)      2
!C   BLANK                   3
!C   OTHER                   4
!C   MIXED                  -1
!C   FORTRAN NUMBER         -2
!CDUMMY ARGUMENT:
!CSTR   THE STRING
!C
      CHARACTER*(*) STR

      ITYPES=0
      IEND=LENSTR(STR)
      IF(IEND.LE.0) GO TO 90

      ITYPES=ITYPE(STR(1:1))
      IF(IEND.EQ.1) GO TO 90

      DO 10 I=2,IEND
         IF(ITYPES.NE.ITYPE(STR(I:I))) GO TO 20
10       CONTINUE
      GO TO 90
20    CONTINUE
      DO 30 I=1,IEND
         IF(STR(I:I).EQ.' ') GO TO 30
         IF(STR(I:I).EQ.'+'.OR.STR(I:I).EQ.'-') GO TO 30
         IF(STR(I:I).GE.'0'.AND.STR(I:I).LE.'9') GO TO 30
         IF(STR(I:I).EQ.'.') GO TO 30
         IF(STR(I:I).EQ.'E') GO TO 30
         ITYPES=-1
         GO TO 90
30       CONTINUE
      ITYPES=-2
90    RETURN
      END
      INTEGER FUNCTION ITYPE(CHARA)
!C
!CPROGRAM UNIT 14
!CRETURNS THE CHARACTER TYPE CODE OF THE INPUT CHARACTER
!C(ONE CHARACTER STRING).
!C   NUMERIC (0-9)           1
!C   ALPHA (A-Z OR a-z)      2
!C   BLANK                   3
!C   OTHER                   4
!CDUMMY ARGUMENT:
!CCHARA CHARACTER*1 STRING
!C
      CHARACTER*1 CHARA

      ITYPE=4
      IF(CHARA.GE.'0'.AND.CHARA.LE.'9') ITYPE=1
      IF(CHARA.GE.'A'.AND.CHARA.LE.'Z') ITYPE=2
      IF(CHARA.GE.'a'.AND.CHARA.LE.'z') ITYPE=2
      IF(CHARA.EQ.' ') ITYPE=3
      RETURN
      END
      SUBROUTINE DSCADX(DZ,IDX,IPWR)
!C
!CPROGRAM UNIT 15
!CCOMPUTE IDX IN THE RANGE 3 TO 25.
!CIPWR IS POWER OF 10 TO GET BACK TO ORIGINAL.
!CDUMMY ARGUMENTS:
!CDZ    INPUT UNCERTAINTY
!CIDX   COMPUTED UNCERTAINTY (ASSIGNED)
!CIPWR  POWER OF 10 SO THAT DZ=IDX*10**IPWR (ASSIGNED)
!C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      REAL DY

!C---  DOUBLE PRECISION ENTRY
      D=DZ
      GO TO 1

!C---  SINGLE PRECISION ENTRY
      ENTRY SSCADX(DY,IDX,IPWR)
      D=DY

1     CONTINUE
      IPWR=0

!C---  D < 3.0, MULTIPLY BY 10.
10    IF(D.LT.3.0) THEN
         D=D*10.0
         IPWR=IPWR-1
         GO TO 10
      ENDIF

!C---  D > 25.0, DIVIDE BY 10.
20    IF(D.GT.25.0) THEN
         D=D/10.0
         IPWR=IPWR+1
         GO TO 20
      ENDIF

!C---  D IN RANGE 3 TO 25, ROUND AND FIX.
      IDX=INT(D+0.9)
      RETURN
      END
      SUBROUTINE DSCALX(Z,IX,IPWR)
!C
!CPROGRAM UNIT 16
!CCOMPUTE IX BASED ON Z AND IPWR
!CDUMMY ARGUMENTS:
!CZ     INPUT VALUE
!CIX    RENORMALIZED Z (ASSIGNED)
!CIPWR  POWER OF 10 SO THAT Z=IX*10**IPWR
!C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      REAL Y

!C---  DOUBLE PRECISION ENTRY
      X=Z
      GO TO 1

!C---  SINGLE PRECISION ENTRY
      ENTRY SSCALX(Y,IX,IPWR)
      X=Y

!C---  SCALE AND FIX X.
1     XX=X*(10D0**(-IPWR))
      IX=INT(XX+SIGN(0.5D0,XX))
      RETURN
      END
      SUBROUTINE SCAL10(IX,IDX,IPWR)
!C
!CPROGRAM UNIT 17
!CIF IDX = 10 OR 20 AND IX A MULTIPLE OF 10,
!CREDUCE IX, IDX, BY 10 AND CHANGE IPWR
!CDUMMY ARGUMENTS:
!CIX    INPUT VALUE (ASSIGNED)
!CIDX   INPUT UNCERTAINTY (ASSIGNED)
!CIDX   INPUT EXPONENT SO THAT VALUE OF X=IX*10**IPWR (ASSIGNED)
!C

      IF((MOD(IX,10).EQ.0).AND.(MOD(IDX,10).EQ.0) ) THEN
         IX=IX/10
         IDX=IDX/10
         IPWR=IPWR+1
      ENDIF
      RETURN
      END
      SUBROUTINE DCALC(Z,DZ,X,DX,Y,DY)
!C
!CPROGRAM UNIT 18
!CCOMPUTE Z AND UNCERTAINTY OF Z FROM X,DX AND Y,DY.
!CSEPARATE ENTRIES FOR ADDITION, SUBTRACTION, MULTIPLICATION
!CAND DIVISION. SINGLE PRECISION ENTRIES USE A,B AND C.
!CDUMMY ARGUMENTS:
!CZ     RESULTING VALUE (ASSIGNED)
!CDZ    RESULTING UNCERTAINTY (ASSIGNED)
!CX,Y   INPUT VALUES
!CDX,DY INPUT UNCERTAINTIES
!C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      REAL A,DA,B,DB,C,DC,CC,DCC

!C---  THIS ENTRY IS DUMMY.
      RETURN

      ENTRY DADD(Z,DZ,X,DX,Y,DY)
      ZZ=X+Y
      DZZ=SQRT(DX*DX+DY*DY)
      Z=ZZ
      DZ=DZZ
      RETURN

      ENTRY SADD(C,DC,A,DA,B,DB)
      CC=A+B
      DCC=SQRT(DA*DA+DB*DB)
      C=C!C
      DC=DCC
      RETURN

      ENTRY DSUB(Z,DZ,X,DX,Y,DY)
      ZZ=X-Y
      DZZ=SQRT(DX*DX+DY*DY)
      Z=ZZ
      DZ=DZZ
      RETURN

      ENTRY SSUB(C,DC,A,DA,B,DB)
      CC=A-B
      DCC=SQRT(DA*DA+DB*DB)
      C=C!C
      DC=DCC
      RETURN

      ENTRY DMUL(Z,DZ,X,DX,Y,DY)
      ZZ=X*Y
      DZZ=0.0
      IF(X.NE.0.0.AND.Y.NE.0.0) DZZ=ZZ*SQRT((DX/X)**2+(DY/Y)**2)
      Z=ZZ
      DZ=DZZ
      RETURN

      ENTRY SMUL(C,DC,A,DA,B,DB)
      CC=A*B
      DCC=0.0
      IF(A.NE.0.0.AND.B.NE.0.0) DCC=CC*SQRT((DA/A)**2+(DB/B)**2)
      C=C!C
      DC=DCC
      RETURN

      ENTRY DDIV(Z,DZ,X,DX,Y,DY)
      ZZ=0.0
      DZZ=0.0
      IF(Y.EQ.0.0) GO TO 10
      ZZ=X/Y
      IF(X.EQ.0.0) GO TO 10
      DZZ=ZZ*SQRT((DX/X)**2+(DY/Y)**2)
10    Z=ZZ
      DZ=DZZ
      RETURN

      ENTRY SDIV(C,DC,A,DA,B,DB)
      CC=0.0
      DCC=0.0
      IF(B.EQ.0.0) GO TO 20
      CC=A/B
      IF(A.EQ.0.0) GO TO 20
      DCC=CC*SQRT((DA/A)**2+(DB/B)**2)
20    C=C!C
      DC=DCC
      RETURN
      END
      SUBROUTINE DCNVFS (Z, DZ, SX, LENX, SDX, LENDX)
!C
!CPROGRAM UNIT 19
!C   CONVERTS THE DOUBLE PRECISION NUMBER Z, WITH OPTIONAL
!C   UNCERTAINTY DZ, INTO STRING FORMAT.  ONE OF FOUR FORMATS
!C   IS SELECTED BASED ON THE VALUES OF DX AND LENDX.
!C   DUMMY ARGUMENTS:
!C   Z     IS THE INPUT DOUBLE PRECISION NUMBER TO BE CONVERTED.
!C   DZ    IS THE INPUT DOUBLE PRECISION UNCERTAINTY IN Z.
!C   SX *  IS THE OUTPUT STRING FOR Z (AND IN FORMAT 2 ALSO DZ).
!C   LENX  IS THE INPUT LENGTH SPECIFIER FOR SX.
!C   SDX * IS THE OUTPUT STRING FOR DZ (FORMATS 1 AND 3 ONLY).
!C   LENDX IS THE INPUT LENGTH SPECIFIER FOR SDX (FORMATS 1 AND 3).
!C         OR A FORMAT FLAG (FORMAT 2 AND 4).
!C   * ASSIGNED IN ROUTINE
!C
!C   FORMAT 1:  DX > 0.0, LENDX > 0.
!C      SX AND SDX ARE SET.
!C      SDX WILL BE IN THE RANGE 1 TO 25.
!C      SX WILL BE SET AS APPROPRIATE FOR THE SPECIFIED UNCERTAINTY.
!C
!C   FORMAT 2:  DX > 0.0, LENDX <= 0.
!C      SX ONLY IS SET, SDX IS NOT MODIFIED.
!C      X AND DX ARE FORMATTED INTO SX.  THE UNCERTAINTY IS NOT
!C      CONSTRAINED TO THE RANGE 1 TO 25 IS DX > 25.0.
!C
!C   FORMAT 3:  DX = 0.0, LENDX >= 0.
!C      SX AND SDX ARE SET.
!C      SX WILL BE SET USING 4 SIGNIFICANT DIGITS.
!C      SDX WILL BE BLANKED OUT TO A LENGTH OF LENDX.
!C
!C   FORMAT 4:  DX = 0.0, LENDX < 0.
!C      SX ONLY IS SET, SDX IS NOT MODIFIED.
!C      SX WILL BE SET USING -LENDX SIGNIFICANT DIGITS.
!C
!C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      REAL Y,DY
      CHARACTER*(*) SX,SDX
      CHARACTER*30 TEMP

!C---  DOUBLE PRECISION ENTRY
      X=Z
      DX=DZ
      GO TO 1

!C---  SINGLE PRECISION ENTRY
      ENTRY SCNVFS (Y, DY, SX, LENX, SDX, LENDX)
      X=Y
      DX=DY

1     IF(DX.GT.0.0.AND.LENDX.GT.0) THEN
!C---     FORMAT 1:  SX, SDX (1, 25).
         CALL DSCADX(DX, IDX, IPWR)
         CALL DSCALX(X, IX, IPWR)
         if(20*dx.le.abs(x)) CALL SCAL10(IX, IDX, IPWR)
         CALL CNVIXS(IX, IPWR, SX, LENX)
         CALL CNVIS(IDX, SDX, LENDX)
         GOTO 90
      ENDIF

      IF(DX.GT.0.0.AND.LENDX.LE.0) THEN
!C---     FORMAT 2:  SX ONLY (IDX INCLUDED).
         IF (DX .GT. 25.0) THEN
            IDX = DX + 0.9
            IX = X + SIGN(0.5D0, X)
            IPWR = 0
         ELSE
            CALL DSCADX(DX,IDX,IPWR)
            CALL DSCALX(X, IX, IPWR)
         ENDIF
         CALL CNVIXS(IX, IPWR, SX, LENX)
         IF(SX(1:1).EQ.'*') GO TO 90
         CALL SQUEEZ(SX,' ')
         CALL CNVIS(IDX, TEMP, 0)
         CALL PUTIN(TEMP,1,' ',1)
         CALL APPEND(SX, TEMP)
         GOTO 80
      ENDIF

      IF(DX.LE.0.0) THEN
!C---     FORMAT 3:  SX HAS 4 SIG. DIGITS, SDX BLANKED.
!C---     FORMAT 4:  SX HAS -LENDX SIG. DIGITS, SDX UNTOUCHED.
         ISIG = 4
         IF (LENDX .LT. 0) ISIG = -LENDX
!C---     FIND PROPER IPWR.
         T = 0.0
         IF (X.NE.0.0) T = LOG10(ABS(X))
         IF (T.LT.0.0) T = T - 1.0
         IPWR = INT(T) - ISIG + 1
         CALL DSCALX(X, IX, IPWR)
         CALL CNVIXS(IX, IPWR, SX, LENX)
         IF(LENDX.GE.0) THEN
            SDX='                                                   '
            CALL PADRT(SDX, LENDX)
            GOTO 90
         ENDIF
      ENDIF

80    CONTINUE
      IF(LENSTR(SX).GT.LENX) THEN
         SX='********************'
      ENDIF


90    CONTINUE
      call lbsup(SX)
      call sresto(SX)
      call lbsup(SDX)
      call sresto(SDX)

      RETURN
      END
      SUBROUTINE CNVIS(N,STR,LD)
!C
!CPROGRAM UNIT 20
!CCONVERTS THE INTEGER N INTO A RIGHT JUSTIFIED STRING, STR,
!CWITH LENGTH LD. IF LD=0, THE STRING LENGTH IS THAT OF THE
!CSQUEEZED STRING. IF N IS TOO LARGE FOR LD CHARACTERS OR FOR
!CMAX. LENGTH OF STR, STARS FILL STR.
!CMAXIMUM INTEGER LENGTH IS 20 CHARACTERS.
!CDUMMY ARGUMENTS:
!CN     INPUT INTEGER
!CSTR   OUTPUT STRING (ASSIGNED)
!CLD    CURRENT LENGTH OF STR
!C
      CHARACTER*(*) STR
      CHARACTER WORK*20

!C---  WRITE NUMBER IN WORK STRING
      WRITE(WORK,100) N
100   FORMAT(I20)

!C---  SQUEEZE OUT BLANKS, FIND LENGTH OF WORK.
      CALL SQUEEZ(WORK,' ')
      LCW=LENSTR(WORK)
      LMS=LEN(STR)
      IF((LCW.GT.LD.AND.LD.GT.0).OR.LCW.GT.LMS) THEN
!C---     NOT ENOUGH SPACE FOR NUMBER.
         STR='********************'
         IF(LD.GT.0) CALL SRESET(STR,LD)
         GO TO 90
      ENDIF
      STR=WORK
      IF(LD.GT.0) CALL PADLFT(STR,LD)
      IF(LD.LE.0) CALL SRESET(STR,LCW)
90    RETURN
      END
      SUBROUTINE CNVIXS (IX, IPWR, SX, LENX)
!C
!CPROGRAM UNIT 21
!CCONVERT IX WITH SCALE FACTOR IPWR TO A STRING SX OF LENGTH
!CLENX.  IF THE STRING SPACE IS TOO SMALL, RETURN STARS (*).
!CIF IPWR > 0 USE EXPONENTIAL FORMAT.
!CIF IX > 0 AND IX * 10 ** IPWR < 1E-4 USE EXPONENTIAL FORMAT.
!CDUMMY ARGUMENTS:
!CIX    INPUT INTEGER
!CIPWR  POWER OF 10 TO NORMALIZE IX TO CORRECT VALUE
!CSX    OUTPUT STRING (ASSIGNED)
!CLENX  CURRENT LENGTH OF OUTPUT STRING SX
!C
      CHARACTER SX*(*),TEMP*30,WORK*10
!C
!C---  CONVERT IX TO STRING MODE.
!C
      CALL CNVIS(IX, TEMP, 0)
!C
!C---  NEG IS CONTROL FOR NEGATIVE NUMBER (LEAVE SPACE FOR SIGN).
!C
      NEG = 0
      IF (IX .LT. 0) NEG = 1
!C
!C---  SPECIAL FORMATTING BASED ON IPWR.
!C
      IF(IPWR.EQ.0) THEN
!C---     IPWR = 0, RETURN INTEGER FORMAT.
         GO TO 80
      ENDIF

      IF(IPWR.GT.0) THEN
!C---     IPWR > 0, RETURN DIG . FRACT E EXP.
!C---     COMPUTE EXPONENT VALUE.
         JPWR=LENSTR(TEMP)-1+IPWR-NEG
!C---     ADD DECIMAL POINT TO NUMBER
         CALL PUTIN(TEMP,2+NEG,'.',1)
!C---     ADD EXPONENT TO END OF STRING.
         CALL APPEND(TEMP,'E+')
         CALL CNVIS(JPWR,WORK,0)
         CALL APPEND(TEMP,WORK)
         GO TO 80
      ENDIF

!C---  IPWR < 0.
      X=REAL(IX)*10.0**IPWR
      IF(ABS(X).LT.1.E-4) THEN
!C---     USE EXPONENTIAL FORMAT FOR VERY SMALL NUMBERS, EXCEPT 0.0.
         IF(IX.EQ.0) THEN
            TEMP='0.0'
            CALL SRESET(TEMP,3)
            GO TO 80
         ENDIF
!C---     COMPUTE EXPONENT VALUE
         JPWR=LENSTR(TEMP)-1+IPWR-NEG
!C---     ADD DECIMAL POINT TO NUMBER
         CALL PUTIN(TEMP,2+NEG,'.',1)
!C---     ADD EXPONENT TO END OF TEMP
         CALL APPEND(TEMP,'E')
         CALL CNVIS(JPWR,WORK,0)
         CALL APPEND(TEMP,WORK)
         GO TO 80
      ENDIF

      LCT=LENSTR(TEMP)
      IF(LCT-NEG.GT.-IPWR) THEN
!C---     DIG.FRACTION
!C---     FIND WHERE DECIMAL POINT BELONGS AND INSERT IT.
         CALL PUTIN(TEMP,LCT+1+IPWR,'.',1)
         GO TO 80
      ELSE
!C---     0.0'S FRACTION
!C---     FIND NUMBER OF LEADING ZEROS AND INSERT.
         IZER=-IPWR-LCT+NEG
         IF(IZER.GE.1) THEN
         DO 10 I=1,IZER
            CALL PUTIN(TEMP,I+NEG,'0',1)
10          CONTINUE
         ENDIF
!C---     INSERT DECIMAL POINT AND FIRST ZERO.
         CALL PUTIN(TEMP,1+NEG,'0.',2)
      ENDIF

80    CONTINUE
!C---  CHECK THAT LENSTR(TEMP) <= LENX. IF SO PUT TEMP IN SX,
!C---  IF NOT FILL SX WITH STARS.
      IF(LENSTR(TEMP).LE.LENX) THEN
         SX=TEMP//'                                                '
      ELSE
         SX='********************'
      ENDIF
      CALL PADLFT(SX,LENX)

90    RETURN
      END
      INTEGER FUNCTION IGET(STR)
!C
!CPROGRAM UNIT 22
!CCONVERTS THE INPUT STRING INTO AN INTEGER (FUNCTION VALUE).
!CONLY DIGITS PRECEEDING THE FIRST NON-NUMERIC CHARACTER,
!CBLANK OR END OF STRING ARE CONVERTED.
!CDUMMY ARGUMENT:
!CSTR   STRING CONTAINING AN INTEGER
!C
      CHARACTER*(*) STR
      CHARACTER*10 NUM
      LOGICAL NEG
      DATA NUM /'0123456789'/

      NEG=.FALSE.
      IGET=0
      L=LENSTR(STR)
      IF(L.LE.0) GO TO 90
      ISTART=1
      IF(STR(1:1).EQ.'-') THEN
         NEG=.TRUE.
         ISTART=2
      ENDIF
      IF(STR(1:1).EQ.'+') ISTART=2
      IF(ISTART.GT.L) GO TO 90
         DO 10 I=ISTART,L
            DO 5 J=1,10
               IF(STR(I:I).EQ.NUM(J:J)) THEN
                  IGET=10*IGET+(J-1)
                  GO TO 10
               ENDIF
5              CONTINUE
!C---  END OF NUMBER
            GO TO 80
10          CONTINUE
!C---  IS NUMBER NEGATIVE?
80    IF(NEG) IGET=-IGET
90    RETURN
      END
      SUBROUTINE WEND(Z,SX,LENX,SDX,LENDX,SLIM,UNIT)
!C
!CWrites value (Z(1) and asymmetric uncertainty Z(2),Z(3)
!Cto strings SX and SDX. If LENDX<6, only first uncertainty (Z(2))
!Cis written. If Z(2)=0.0, uncertainty is replaced by SLIM.
!CIf UNIT non-blank, it is appended to value.
!C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE PRECISION Z(3)
      CHARACTER*(*) SX,SDX,SLIM,UNIT
      CHARACTER*10 TEMP1,TEMP2

      X=Z(1)
      DX1=Z(2)
      DX2=Z(3)
      SX=' '
      SDX=' '

      if(dx1.gt.0.0) then
!C   Scale according to smaller of dx1 and dx2:
         t=dx1
         if (dx2.lt.t.and.dx2.gt.0) t=dx2
         CALL DSCADX(t,IDX,IPWR)
         CALL DSCALX(DX1,IDX,IPWR)
         CALL DSCALX(X,IX,IPWR)
         if(20*dx1.le.abs(x)) CALL SCAL10(IX, IDX, IPWR)
         CALL CNVIXS(IX,IPWR,SX,LENX)
         CALL CNVIS(IDX,SDX,LENDX)
         call lbsup(SX)
         call tbsup(SX)
         call append(SX,UNIT)
         call lbsup(SDX)

!C   Finished if dx2=0, LENDX<6 or DX1=DX2
         if(dx2.le.0.or.LENDX.lt.6.or.DX1.eq.DX2) go to 90

!C   Calculate other error limit
         temp1='+'//SDX
         call tbsup(temp1)
         CALL DSCALX(DX2,IDX2,IPWR)
         CALL CNVIS(IDX2,SDX,LENDX)
         call lbsup(SDX)
         temp2='-'//SDX
         call append(temp1,temp2)
         sdx=temp1

      else
!C   Determine number of 2 significant digits in X
!C   SDX is replaced by SLIM
         ISIG = 2
         if(X.ge.100.and.X.lt.1000) ISIG=3
         if(X.ge.1000.and.X.lt.10000) ISIG=4
!C   FIND PROPER IPWR.
         T = 0.0
         IF (X.NE.0.0) T = LOG10(ABS(X))
         IF (T.LT.0.0) T = T - 1.0
         IPWR = INT(T) - ISIG + 1
         CALL DSCALX(X, IX, IPWR)
         CALL CNVIXS(IX, IPWR, SX, LENX)
         call lbsup(SX)
         call tbsup(SX)
         call append(SX,UNIT)
         SDX=SLIM
      endif

90    CONTINUE
      call sresto(SX)
      call sresto(SDX)

      RETURN
      END
      SUBROUTINE DCNVSF(STR1,STR2,Z,DZ)
!C
!CPROGRAM UNIT 23
!CCONVERT STR1 AND STR2
!CINTO TWO DOUBLE PRECISION NUMBERS Z AND DZ, RESPECTIVELY.
!CSTR1 AND STR2 SHOULD BE <= 30 CH:S LONG, OR STRINGS WILL BE
!CTRUNCATED.
!CDUMMY ARGUMENTS:
!CSTR1  INPUT STRING CONTAINING VALUE
!CSTR2  INPUT STRING CONTAINING UNCERTAINTY
!CZ     OUTPUT VALUE (ASSIGNED)
!CDZ    OUTPUT UNCERTAINTY (ASSIGNED)
!C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      REAL Y,DY
      LOGICAL SINGLE
      CHARACTER*(*) STR1,STR2
      CHARACTER*30 S1,S2,SDUM

!C---  DOUBLE PRECISION ENTRY
      SINGLE=.FALSE.
      GO TO 1

!C---  SINGLE PRECISION ENTRY
      ENTRY SCNVSF(STR1,STR2,Y,DY)
      SINGLE=.TRUE.

1     S1=STR1
      S2=STR2

!C---  INITIAL OUTPUT VALUES
      X=0.0
      DX=0.0

!C---  SUPPRESS ALPHAS AND SQUEEZE OUT BLANKS
      CALL SUPALF(S1)
      CALL SUPALF(S2)
      CALL SQUEEZ(S1,' ')
!C---  TAKE AWAY POSSIBLE 'E':S FROM S2
      CALL SQUEEZ(S2,'E')
      CALL SQUEEZ(S2,' ')
      IF(LENSTR(S1).LE.0) GO TO 90

!C---  LOOK FOR '.' OR 'E' IN S1
      IDOT=INDEX(S1,'.')
      IE  =INDEX(S1,'E')

!C---  GET IPWR
      IPWR=0
      IF(IE.GT.0) THEN
         SDUM=S1(IE+1:)
         IPWR=IGET(SDUM)
      ENDIF

!C---  GET IFRACT AND FIND NUMBER OF DIGITS IN IFRACT.
      IFRACT=0
      K=0
      IF(IDOT.GT.0) THEN
!C---     CASE ONE NO EXPONENT
         IF(IE.LE.0) SDUM=S1(IDOT+1:)
!C---     CASE TWO EXPONENT GIVEN, CHOP IT OFF
         IF(IE.GT.0) SDUM=S1(IDOT+1:IE-1)
         CALL SQUEEZ(SDUM,' ')
         K=LENSTR(SDUM)
         IFRACT=IGET(SDUM)
      ENDIF

!C---  GET INTEG
      INTEG=IGET(S1)

!C---  COMPUTE X
      POWER=1.0
!C---  DIVIDE IFRACT BY 10**K
      IF(IDOT.GT.0) POWER=10.0D0**(-K)
      FRACT=REAL(IFRACT)*POWER
      X=DBLE(INTEG)+SIGN(FRACT,DBLE(INTEG))
      DPWR=1.0
      IF(IPWR.NE.0) DPWR=10.0D0**IPWR
      X=X*DPWR

!C---  COMPUTE DX
      DX=DBLE(IGET(S2))*POWER*DPWR

90    IF(SINGLE) THEN
         Y=X
         DY=DX
         GO TO 99
      ENDIF
      Z=X
      DZ=DX
99    RETURN
      END

      SUBROUTINE READT(STR,T,DT)
!C
!CPROGRAM UNIT 27
!CREADS HALF-LIFE FROM STRING STR. IF UNIT IS GIVEN,
!CTHE RESULT IS CONVERTED TO SECONDS.
!C** NOTE: THERE MUST BE AT LEAST ONE SPACE BETWEEN UNIT AND ERROR.
!CIF STR IS NOT IN 2-CARD FORMAT THE CALL SHOULD BE E.G.
!C  CALL READT(CARD(40:49)//' '//CARD(50:55),T,DT)
!CIN ORDER TO GET THE NECESSARY SPACE.
!C
!CDUMMY ARGUMENTS:
!CSTR   INPUT STRING WITH HALF LIFE IN FORMAT: VALUE UNIT ERROR
!CT     HALF-LIFE VALUE IN SECONDS (ASSIGNED)
!CDT    UNCERTAINTY IN HALF-LIFE (ASSIGNED)
!C
      CHARACTER STR*(*),WORK*50,UNIT*10,TUNITS(17)*3
      REAL TFACT(17)
      DATA TUNITS /'AS ','FS ','PS ','NS ','US ','MS ','S  ', &
                  'M  ','H  ','D  ','Y  ','KY ','MY ','GY ','EV ', &
                  'KEV','MEV'/
!C
!C1 YEAR IS 365.256 DAYS
!C
!CT1/2(S)=LN(2)*HBAR/WIDTH(EV) WHERE
!CLN(2)*HBAR=C=4.5624*10**-16 EV*S --> 1/C=2.1918*10**15
!C
      DATA TFACT /1.E-18,1.E-15,1.E-12,1.E-9,1.E-6,1.E-3,1.,60., &
      3600.,8.64E+4,3.155812E+7,3.155812E+10,3.155812E+13, &
      3.155812E+16,2.1918E+15,2.1918E+18,2.1918E+21/

      WORK=STR
!C---  GET VALUE AND UNIT
      CALL READ2(WORK,0,' ',IFOUND,UNIT,T,DT)
!C---  INTERPRET UNIT, CONVERT TO SECONDS.
      LCU=LENSTR(UNIT)
      IF(LCU.GT.3.OR.LCU.LE.0) GO TO 90
!C---  MAKE UNIT 3 CHARACTERS LONG
      IF(LCU.EQ.1) UNIT(2:2)=' '
      IF(LCU.LE.2) UNIT(3:3)=' '
      DO 10 I=1,17
         IF(UNIT(1:3).EQ.TUNITS(I)) THEN
            T=T*TFACT(I)
            DT=DT*TFACT(I)
!C---        SPECIAL TREATMENT FOR EV, KEV, MEV
            IF(I.GE.15) THEN
               IF(T.GT.0.0) THEN
                  RELERR=DT/T
                  T=1.0/T
                  DT=T*RELERR
               ENDIF
               GO TO 90
            ENDIF
            GO TO 90
         ENDIF
10       CONTINUE
90    RETURN
      END
!C**************************************
      Subroutine fendU(ic1,ic2,x,mx,nx,unit)
!c  Decode ENSDF data into numeric data
!c  Array x returns value and asymmetric uncertainties
!c  String NX returns non-numerical uncertainty
!c  String MX returns '  ?' or ' ()'
!c  String UNIT returns valid unit

      implicit DOUBLE PRECISION (a-h,o-z)
      Dimension x(3)
      Character*(*) ic1,ic2,mx,nx,unit
      character*20 jc,kc,jbl
      character units3(17)*3

      DATA UNITS3 /' S ',' M ',' H ',' D ',' Y ', &
                  ' MS',' US',' NS',' PS',' FS',' AS', &
                  ' KY',' MY',' GY',' EV','KEV','MEV'/

      k=len(ic1)
      kk=len(ic2)
      mx='   '
      nx='  '
      unit=' '
      x(1)=0.0
      x(2)=0.0
      x(3)=0.0
!CBlank value - return
      if(IC1.EQ.' ') return

      jc=ic1
      jbl=' '
      kc=' '

!CFind unit, put in string UNIT and blank it out from J!C

      do i=1,17
         n=index(jc,units3(i))
         if (n.gt.0) then
            unit=units3(i)
            jc(n:n+2)='   '
            goto 5
         endif
      enddo

5     continue

!c Check for nonnumeric quantities
1      n=index(jc,'+')

!CPrevious character is E (exponent)
!c$$   if(n.eq.0.or.jc(n-1:n-1).eq.'E') go to 10
      if(n.le.1.or.jc(n-1:n-1).eq.'E') go to 10

!CFollowing character is a number
      if(numb(jc(n+1:n+1)).ne.0) go to 15
      mx(1:2)=jc(n:n+1)
      jc(n:n+1)='  '
      if(n+2.gt.k.or.jc(n+2:n+2).eq.' ') go to 100
      mx(3:3)=jc(n+2:n+2)
      jc(n+2:n+2)=' '
      go to 100
15    continue
      mx(2:3)=jc(n-1:n)
      jc(n-1:n)='  '
      if(n-2.le.0.or.jc(n-2:n-2).eq.' ') go to 100
      mx(1:1)=jc(n-2:n-2)
      jc(n-2:n-2)=' '
      go to 100
10    continue
      n=index(jc,'?')
      if(n.eq.0) go to 20
      jc(n:n)=' '
      mx='  ?'
      go to 100
20    continue
      n=index(jc,'(')
      if(n.eq.0) go to 30
      mx=' ()'
      jc(n:n)=' '
      n=index(jc,')')
      if(n.gt.0) jc(n:n)=' '
      go to 100
30    continue
      n=index(jc,'X')
      if(n.eq.0) go to 40
      jc(n:n+2)='0+X'
      go to 1
40    continue
      n=index(jc,'Y')
      if(n.eq.0) go to 45
      jc(n:n+2)='0+Y'
      go to 1
45    continue
      n=index(jc,'SN')+index(jc,'SP')
      if(n.eq.0) go to 100
      jc(n:n+1)='  '
      mx(2:3)=jc(n:n+1)
      go to 1
100   continue
      read(jc(1:k),1000,err=101) x(1)
1000  format(bn,F20.0)
      go to 150
101   continue
      iii=1
      do 110 i=1,k
      if(numb(jc(i:i)).eq.1.or.jc(i:i).eq.'.') go to 110
      jc(i:i)=' '
      if(iii.lt.3) mx(iii:iii)=jc(i:i)
      iii=iii+1
110   continue
150   continue

!c  Check uncertainties for alphanumerics
      jj=0
      if(ic2(1:kk).eq.jbl(1:kk)) go to 99
      do 50 i=1,kk
         if(ic2(i:i).eq.'-'.or.ic2(i:i).eq.'+') go to 50
         if(ic2(i:i).ne.' '.and.numb(ic2(i:i)).eq.0) go to 200
         if(numb(ic2(i:i)).eq.1) jj=i
50    continue

      if(jj.eq.0) go to 99
      go to 500

200   continue
      nx=ic2(i:i+1)
      go to 99

500   continue

!c   Process uncertainties
      mm=k
      nq=0
      n=0
      n=index(jc,'E')
      n=n+index(jc,'e')
      if(n.eq.0) go to 505
      mm=n-1
      kc(n:k)=jc(n:k)
505   continue
      m1=2
506   continue
      do 510 i=1,mm
      n=mm-i+1
      if(jc(n:n).eq.' ') kc(n:n)=' '
      if(jc(n:n).eq.'.') kc(n:n)='.'
      if(jc(n:n).eq.' '.or.jc(n:n).eq.'.') go to 510
      kc(n:n)='0'
      if(nq.eq.1) go to 510
512   continue
      if(jj.le.0) go to 513
      if(ic2(jj:jj).ne.' ') go to 511
      jj=jj-1
      if(jj.gt.0) go to 512
      go to 513
511   continue
      kc(n:n)=ic2(jj:jj)
      jj=jj-1
513   continue
      if(jj.le.0.or.ic2(jj:jj).eq.'-'.or.ic2(jj:jj).eq.'+') nq=1
510   continue
      read(kc(1:k),1000) x(m1)
      if(m1.eq.3.or.x(3).gt.1.0D-20) go to 99
      if(ic2(jj:jj).ne.'-'.and.ic2(jj:jj).ne.'+') go to 98
      if(ic2(jj:jj).eq.'+') go to 515
      x(3)=x(2)
      x(2)=0.0
      ml=2
      go to 516
515   continue
      m1=3
516   continue
      kc=jbl
      jj=jj-1
      nq=0
      go to 500
98    continue
      x(3)=x(2)
99    continue
      return
      end
!C***************************************
      SUBROUTINE READ2(STR,LOOK,STR1,IFOUND,UNIT,X,DX)
!C
!CPROGRAM UNIT 28
!CREADS VALUE AND ERROR OF NUMBER IN STR
!CFOLLOWING THE STRING STR1. IFOUND=1 SIGNALS THAT STR1
!CIS FOUND. IF LOOK=0, NUMBER IS READ DIRECTLY FROM STR WITHOUT
!CSEARCHING FOR STR1. STRING UNIT CONTAINS UNIT IF GIVEN.
!CDUMMY ARGUMENTS:
!CSTR    INPUT STRING IN FORMAT VALUE UNIT ERROR (2 CARD FORMAT)
!CLOOK   INDICATES WHETHER STRING STR1 SHOULD BE LOOKED FOR OR NOT
!CSTR1   OPTIONAL STRING TO BE LOOKED FOR (E.G. 'KC=')
!CIFOUND INDICATES IF STR1 IS FOUND IN STR (ASSIGNED)
!CUNIT   UNIT READ IMMEDIATELY AFTER NUMBER (ASSIGNED)
!CX      VALUE READ (ASSIGNED)
!CDX     UNCERTAINTY READ (ASSIGNED)
!C
      CHARACTER*(*) STR,STR1,UNIT,WORK*30,DUMSTR*80

      X=0.0
      DX=0.0
      IF(LOOK.EQ.0) GO TO 30
      IFOUND=0
      UNIT='                                                        '
      IPOINT=INDEX(STR,STR1)
      IF(IPOINT.GT.0) THEN
         IFOUND=1
         WORK=STR(IPOINT+LEN(STR1):)
         GO TO 35
      ENDIF
      GO TO 90
30    CONTINUE
      WORK=STR
35    CONTINUE
!C---  FIND END OF STRING ($ DELIMITER)
      IEND=INDEX(WORK,'$')
      DUMSTR=WORK
      IF(IEND.GT.0) WORK=DUMSTR(1:IEND)
!C---  FIND END OF FIRST NUMBER AND CHECK IF UNIT
      CALL LBSUP(WORK)
      IBLANK=INDEX(WORK,' ')
      IF(IBLANK.LE.0) THEN
         IBLANK=30
         UNIT='                                                      '
      ELSE
         UNIT=WORK(IBLANK+1:)
         CALL LBSUP(UNIT)
         IBLAN1=INDEX(UNIT,' ')
         IF(IBLAN1.GT.0) THEN
            DUMSTR=UNIT
            UNIT=DUMSTR(1:IBLAN1)
            CALL TBSUP(UNIT)
         ENDIF
      ENDIF
      CALL SCNVSF(WORK(1:IBLANK-1),WORK(IBLANK:30),X,DX)
90    RETURN
      END
      COMPLEX FUNCTION GAMA(X)
!C
!CPROGRAM UNIT 29 A
!CZ = GAMMA(X) FOR ALL VALUES OF X (Z, X COMPLEX).
!C*** THIS IS AN UNCHANGED BROOKHAVEN VERSION 5(16) S.P. ****
!CDUMMY ARGUMENT:
!CX     SINGLE PRECISION COMPLEX INPUT VALUE
!C
      COMPLEX X
      COMPLEX XTMP, GAMZ
      INTEGER J, N
      REAL FJ, FN
!C
      IF ((REAL(X) .LT. 0.) .OR. (AIMAG(X) .LT. 0.)) GOTO 10
          GAMA = GAMZ(X)
          GOTO 99
!C
   10 IF (REAL(X) .LT. 0.) GOTO 20
          GAMA = CONJG(GAMZ(CONJG(X)))
          GOTO 99
!C
   20 N = 1. - REAL(X)
      FN = N
      XTMP = X + FN
      J = 0
      IF (AIMAG(X) .LT. 0.) GOTO 30
          GAMA = GAMZ(XTMP)
          GOTO 40
!C
   30 GAMA = CONJG(GAMZ(CONJG(XTMP)))
!C
   40 FJ = J
      XTMP = X + FJ
      GAMA = GAMA / XTMP
      J = J + 1
      IF (N .GT. J) GOTO 40
!C
   99 RETURN
      END
      COMPLEX FUNCTION GAMZ(X)
!C
!CPROGRAM UNIT 29 B
!C  Z = GAMMA(X) FOR X(REAL), X(IMAG) >= 0.
!CNOTE - GAMZ CALLS GAM1 WHICH MODIFIES ITS ARGUMENT.
!C
      COMPLEX X
      COMPLEX XTMP, XT, Z1, Z2, C, GAM1
      REAL FN, F1N, S, T, PI
      INTEGER J, M
      DATA PI/2.50662827463/
!C
      IF (AIMAG(X) .GT. 1.) GOTO 10
          XT = X
          GAMZ = GAM1(XT)
          GOTO 99
!C
   10 M = AIMAG(X)
      FN = M + 1
      XTMP = X / FN
      XT = XTMP
      Z1 = GAM1(XT)
      F1N = 1. / FN
      J = 1
!C
   20 XTMP = XTMP + F1N
      XT = XTMP
      Z2 = GAM1(XT)
      Z1 = Z1 * Z2
      J = J + 1
      IF (M .GE. J) GOTO 20
!C
      S = (FN ** (REAL(X) - 0.5)) / (PI ** M)
      T = AIMAG(X) * ALOG(FN)
      C = S * CMPLX(COS(T), SIN(T))
      GAMZ = C * Z1
!C
   99 RETURN
      END
      COMPLEX FUNCTION GAM1(X)
!C
!CPROGRAM UNIT 29 !C
!C  Z = GAMMA(X) FOR X(REAL) >= 0, 0 <= X(IMAG) <= 1.
!C
      COMPLEX X
      COMPLEX GAM2
      REAL FJ, FN
      INTEGER J, N
!C
      IF (REAL(X) .GT. 1.) GOTO 10
          GAM1 = GAM2(X)
          GOTO 99
!C
   10 N = REAL(X)
      FN = N
      X = X - FN
      IF (REAL(X) .NE. 0.) GOTO 20
          N = N - 1
          X = X + 1.
!C
   20 GAM1 = GAM2(X)
      J = 0
!C
   30 FJ = J
      GAM1 = (X + FJ) * GAM1
      J = J + 1
      IF (N .GT. J) GOTO 30
!C
   99 RETURN
      END
      COMPLEX FUNCTION GAM2(X)
!C
!CPROGRAM UNIT 29 D
!C  Z = GAMMA(X) FOR 0 <= X(REAL) <= 1, 0 <= X(IMAG) <= 1.
!C
!C  USING PADE-POWER APPROXIMATION OF 1 / GAMMA(X).
!C
      COMPLEX X
      COMPLEX P, Q
      REAL A(9), B(9)
      INTEGER I
      DATA A/ &
         +0.0000000000E+0, +1.0000000000E+0, &
         +1.2536302998E+0, +6.2294126401E-2, &
         -1.9367439704E-1, +9.5294089001E-3, &
         +1.0021677762E-2, -1.7669280217E-3, &
         +7.9027635693E-5/
      DATA B/ &
         +1.0000000000E+0, +6.7641463495E-1, &
         +3.2773507466E-1, +1.0279994528E-1, &
         +2.7018504538E-2, +5.1647208257E-3, &
         +8.7521995448E-4, +9.5129148083E-5, &
         +9.9862892410E-6/
!C
      P = A(9)
      Q = B(9)
      I = 8
!C
   10 P = P * X + A(I)
      Q = Q * X + B(I)
      I = I - 1
      IF (I .GT. 0) GOTO 10
!C
      GAM2 = Q / P
      RETURN
      END
      COMPLEX FUNCTION HYPERG(A, B, X)
!C
!CPROGRAM UNIT 30
!CZ = HYPERGEOMETRIC(A, B, X).
!C*** THIS IS AN UNCHANGED BROOKHAVEN VERSION 5(16) S.P. ***
!CDUMMY ARGUMENTS:
!CA,B,X SINGLE PRECISION COMPLEX INPUT VALUES
!C
!C  ADOPTED FROM 1604 SUBROUTINE OF C.W. NESTOR.
!C
      COMPLEX A, B, X
      COMPLEX APN, BPN, FN, T, TEST
      REAL PRE!C
      INTEGER N
      DATA PREC/1.E-6/
!C
      APN = A
      BPN = B
      FN = 0.
      T = 1.
      HYPERG = 1.
!C
      DO 10 N = 1, 30
          FN = FN + 1.
          T = T * APN * X / FN / BPN
          APN = APN + 1.
          BPN = BPN + 1.
          TEST = T / HYPERG
          HYPERG = HYPERG + T
          IF (CABS(TEST) .LT. PREC) GOTO 99
   10     CONTINUE
      WRITE(6, 100) A, B, X, HYPERG, N
  100 FORMAT(16H ERROR IN HYPERG/ &
         4(5X, 1H(, E20.10, 1H,, E20.10, 1H)/), I10)
      HYPERG = 0.
!C
   99 RETURN
      END
      SUBROUTINE CHARAD(N,IC)
!C  This subroutine imbeds character equivalent of N in IC

      CHARACTER*2 IC
      IC='  '

!C  Maximum N is 3375 (63*52-1+100)
      IF(N.GE.3376) THEN
         call stopit(' Subroutine CHARAD',' ',' ', &
                    ' ** Character array limit exceeded **',N)
      ENDIF

!c  For N=0-99, use N
!C  If N<100 ID=N
      IF(N.LE.99) THEN
         WRITE(IC,'(I2)') N
         RETURN
      ENDIF

!C  Base 63x52, N1=1st character, N2=2nd character
      NN=N-100
      N1=NN/52
      N2=NN-52*N1

!CFor N1 use blank,0-9,A-Z,a-z, i.e. 1+10+26+26=63 combinations

!CFor N1=0 use blank
      IF(N1.GE.1) THEN
!C   For N1=1-10 use 0-9
         IF(N1.LE.10) THEN
            N1=N1-1
            WRITE(IC(1:1),'(I1)') N1
            GO TO 20
         ENDIF

!C   For N1=11-36 use A-Z
         IF(N1.LE.36) THEN
            N1=N1+54
!C   For N1=37-62 use a-z
         ELSEIF(N1.GE.37) THEN
            N1=N1+60
         ENDIF
         IC(1:1)=CHAR(N1)
      ENDIF

20    CONTINUE

!CFor N2 use A-Z,a-z, i.e. 26+26=52 combinations

!CFor N2=0-25 use A-Z
      IF(N2.LE.25) THEN
         N2=N2+65
!CFor N2=26-51 use a-z
      ELSEIF(N2.GE.26) THEN
         N2=N2+71
      ENDIF

      IC(2:2)=CHAR(N2)

      RETURN
      END

      SUBROUTINE ICHARAD(IC,N)
!C  Inverse of subroutine CHARAD: Gets level number from character
!C  code IC

      CHARACTER*2 IC
      INTEGER N

!c  For N= 0-99, IC is N
      if(ic(1:1).eq.' '.or.numb(ic(1:1)).eq.1) then
         if(numb(ic(2:2)).eq.1) then
            read(ic,'(I2)') n
            return
         endif
      endif

!C  Base 63x52, N1=1st character, N2=2nd character
      N1=ICHAR(ic(1:1))
      N2=ICHAR(ic(2:2))

!CCheck validity of N1 and N2 and "renormalise"
!CN1 is blank
      if(N1.eq.32) then
         N1=0

!CN1 is number
      elseif(N1.ge.48.and.N1.le.57) then
         N1=N1-47

!CN1 is A-Z
      elseif(N1.ge.65.and.N1.le.90) then
         N1=N1-54

!CN1 is a-z
      elseif(N1.ge.97.and.N1.le.122) then
         N1=N1-60
      else
         call stopit(' Subroutine ICHARAD',' '//IC,' ', &
                    ' ** Invalid code **',0)
      endif

!CN2 is A-Z
      if(N2.ge.65.and.N2.le.90) then
         N2=N2-65

!CN2 is a-z
      elseif(N2.ge.97.and.N2.le.122) then
         N2=N2-71
      else
         call stopit(' Subroutine ICHARAD',' '//IC,' ', &
                    ' ** Invalid code **',0)
      endif

      N = N1*52 + N2 + 100

      RETURN
      END

      Subroutine GetJPi(JPi,spin,parity,jpimod)

      implicit none
      integer spin,l,i
      character*(*) JPI,card*18
      character*3 mjpi(5),jpimod
      character*1 ierr,pi(5),parity
      integer j(5)

      card=JPi
      call JREAD(card,j,pi,mjpi,l,ierr)
      if(l.eq.1.and.ierr.eq.' ') then
         spin=j(1)
         parity=pi(1)
         jpimod=mjpi(1)
      elseif(l.eq.0) then
         spin=-1
         parity=' '
         jpimod='   '
      else
         spin=-1
         parity=pi(1)
         jpimod='  '
         do i=1,l
            if(mjpi(i)(2:3).eq.'(B'.or.mjpi(i)(2:3).eq.'(P') &
              jpimod=' (P'
            if(pi(i).ne.pi(1)) then
               parity=' '
            endif
         enddo
      endif

      return
      end

      Subroutine PutJPi(JPi,spin,parity,jpimod,iodd)

      implicit none
      integer spin,iodd
      character*(*) JPI,jpimod*3,parity*1
      character*10 jtemp

      JPI=' '
      if(spin.lt.0.and.parity.eq.' ') return

      jtemp='**********'
!C       1234567890
!C       (JJJ/2)(P)

!CWrite spin
      if(spin.ge.0) write(jtemp(2:4),'(I3)') spin
      if(jtemp(2:2).eq.' ')jtemp(2:2)='*'
      if(jtemp(3:3).eq.' ')jtemp(3:3)='*'

!CAdd /2 if applicable
      if(iodd.eq.2.and.spin.ge.0)jtemp(5:6)='/2'

!CAdd parity
      if(parity.ne.' ') jtemp(9:9)=parity

!CAdd appropriate parenthesies
      if(jpimod.eq.' (J'.or.jpimod.eq.' (B') jtemp(1:1)='('
      if(jpimod.eq.' (J'                   ) jtemp(7:7)=')'
      if(jpimod.eq.' (P'.or.jpimod.eq.' (B') jtemp(10:10)=')'
      if(jpimod.eq.' (P'                   ) jtemp(8:8)='('

!CSqueeze out *:s
      call squeez(jtemp,'*')
      call sresto(jtemp)
      JPI=jtemp

      return
      end

      Subroutine GetML(mult,ML,parity,MLMod)
!C
!CGet lowest two multipolarities from ENSDF Mult field and store
!CL-values and parity in ML(2) and parity. MLMod gives uncertainty
!Cstatus.
!C M1/E2:    1,2   '+'
!C Pure E2:  2,-1  '+'
!C M2/E3:    2,3   '-'
!C E0/M1:    0,1   '+'
!C MLMod:    'xyz', x - first L, y - second L, z - parity. For
!C safe assignments x, y, z are ' ', for uncertain '(',
!C for assumed '[', for incorrect '*'.
!C
!C
      implicit none
      integer ML(2),le,lm,ne,nm,NUMB,npar1,npar2
      integer n1,n2,nd,nq,no
      character*(*) mult,parity*1,parm*1,pare*1
      character*3 MLmod

!CReset multipolarity variables
      lm=0
      le=0
      MLmod='   '
      ML(1)=-1
      ML(2)=-1
      parity=' '

!CEmpty field - return
      if(mult.eq.' ') return
!CField containing NOT - return
      if(index(mult,'NOT').gt.0) return

!CFind the first two multipolarities in mult string
      ne=index(mult,'E')
      if(ne.gt.0.and.numb(mult(ne+1:ne+1)).ne.1) ne=0
      nm=index(mult,'M')
      if(nm.gt.0.and.numb(mult(nm+1:nm+1)).ne.1) nm=0

      pare=' '
      parm=' '

!CGet electric multipole and parity
      if(ne.ne.0) then
         read(mult(ne+1:ne+1),'(I1)') le
         if(le.lt.0.or.le.gt.6) goto 90
         if(2*(le/2).eq.le) then
            pare='+'
         else
            pare='-'
         endif
      endif

!CGet magnetic multipole and parity
      if(nm.ne.0) then
         read(mult(nm+1:nm+1),'(I1)') lm
         if(lm.le.0.or.le.gt.6) goto 90
         if(2*((lm+1)/2).eq.lm) then
            parm='-'
         else
            parm='+'
         endif
      endif

!CMake sure parity from El and Ml are consistent
      if(parm.ne.' ') then
         parity=parm
      elseif(pare.ne.' ') then
         parity=pare
      endif
      if(pare.ne.' '.and.parm.ne.' '.and.pare.ne.parm) goto 90

!CGet multipolarities
!CTwo multipolarities
      if(ne.gt.0.and.nm.gt.0) then
         if(le.lt.lm) then
            ML(1)=le
            ML(2)=lm
            n1=ne
            n2=nm
         else
            ML(1)=lm
            ML(2)=le
            n1=nm
            n2=ne
         endif
!COne, magnetIC
      elseif(nm.gt.0) then
         ML(1)=lm
         n1=nm
         n2=99
!COne electrIC
      elseif(ne.gt.0) then
         ML(1)=le
         n1=ne
         n2=99
      endif

!CHandle D, Q and O
      if(ne.eq.0.and.nm.eq.0) then
         nd=index(mult,'D')
         nq=index(mult,'Q')
         no=index(mult,'O')
         if(nd.gt.0) then
            ML(1)=1
            if(nq.gt.0) then
               ML(2)=2
            endif
            n1=nd
            n2=nq
         elseif(nq.gt.0) then
            ML(1)=2
            if(no.gt.0) then
               ML(2)=3
            endif
            n1=nq
            n2=no
         elseif(nq.gt.0) then
            ML(1)=3
            n1=no
            n2=99
         endif
      endif

!CHandle parentheses
      npar1=index(mult,'(')
      if(ML(2).gt.0) then
         if(npar1.gt.0) then
            npar2=index(mult,')')
            if(npar2.le.0) goto 90
            if(npar1.lt.n1.and.npar2.gt.n2) then
               MLMod(1:2)='(('
               if(ne.gt.0.or.nm.gt.0) MLMod(3:3)='('
            elseif(npar1.gt.n1.and.npar2.gt.n2) then
               MLMod(1:2)=' ('
            endif
         endif
      else
         if(npar1.gt.0) MLMod='( ('
      endif

!CHandle IF and []
      if(index(mult,'IF').gt.0.or.index(mult,'[').gt.0) then
         MLMod(1:1)='['
         if(ML(2).gt.0) MLMod(2:2)='['
         if(ne.gt.0.or.nm.gt.0) MLMod(3:3)='['
      endif
      if(ML(2)-ML(1).gt.1) goto 90
      if(index(mult,'+').gt.0.and.ML(2).lt.0) goto 90
      if(index(mult,',').gt.0.and.ML(2).lt.0) goto 90

      goto 99
!CReturn with error
90    continue
      MLMod='***'


99    continue
      return
      end

      Subroutine PutML(mult,ML,parity,MLMod)
!c  Writes multipolarity to mult character string

      implicit none
      integer ML(2),itype1,itype2
      character*(*) mult,MLMod*3,mytype(2)*1
      character*10 temp,parity*1
      data mytype /'E','M'/

      mult=' '
      if(MLmod.eq.'***') then
         mult='**************'
         return
      endif

      if(ML(1).lt.0.and.parity.eq.' ') return

      temp='**********'
!C      1234567890
!C      (E2(+M3)

!CWrite multipolarity
      if(parity.ne.' ') then
         if(parity.eq.'-') then
            itype1=1
            if(2*(ML(1)/2)-ML(1).eq.0) itype1=2
         else
            itype1=2
            if(2*(ML(1)/2)-ML(1).eq.0) itype1=1
         endif
         write(temp(2:2),'(A)') mytype(itype1)
         write(temp(3:3),'(I1)') ML(1)
         if(ML(2).gt.0) then
            itype2=1
            if(itype1.eq.1) itype2=2
            write(temp(7:7),'(I1)') ML(2)
            write(temp(6:6),'(A)') mytype(itype2)
            temp(5:5)='+'
         endif

!CHandle D, Q, and O
      else
         if(ML(1).eq.1) temp(2:2)='D'
         if(ML(1).eq.2) temp(2:2)='Q'
         if(ML(1).eq.3) temp(2:2)='O'
         if(ML(2).gt.0) then
            temp(5:5)='+'
            if(ML(2).eq.2) temp(6:6)='Q'
            if(ML(2).eq.3) temp(6:6)='O'
         endif
      endif

!CHandle brackets/parentheses
      if(MLMod(1:1).eq.'[') then
         temp(1:1)='['
         temp(8:8)=']'
      endif
      if(MLMod(1:1).eq.'(') then
         temp(1:1)='('
         temp(8:8)=')'
      elseif(MLMod(2:2).eq.'(') then
         temp(4:4)='('
         temp(8:8)=')'
      endif

!CSqueeze out *:s
      call squeez(temp,'*')
      call sresto(temp)
      mult=temp

      return
      end

      Subroutine JREAD(icard,j,pi,mjpi,l,ierr)
!C
!C  Subroutine to read ENSDF JPI field and return array of J and PI values
!C  and alphanumeric modifier.
!C
!Cicard contains input spin field, j(5) and pi(5) are the returned
!Cspin and parity values (up to 5), and l is the number of spins
!Creturned.  J=-1 for a blank spin (e.g. only parity given).
!CThe alphanumeric modifiers for each JPI are as follows:
!C         b(J=spin in parenthesis
!C         b(P=parity in parenthesis
!C         b(B-spin and parity in parenthesis
!Cand
!C         b=blank, or ,(separated by commas) or A (separated by AND)
!C           or O (separated by OR).
!CIf the value is systematic, replace ( by [ in the above definitions.
!C
!C
      character*18 icard,IC
      character*3 mjpi(5)
      character*1 ierr,pi(5),p1,p2
      integer j(5),ito,k,j1,j2,kk,jstep
	  logical flag,flagto,odd
	  flagto=.false.
	  odd=.false.
      ic=icard
      ierr=' '
      l=0
      do 5 k=1,5
      j(k)=-1
      pi(k)=' '
5     mjpi(k)='   '
      if(ic.eq.'                  ') return
!C  Check for odd-ball contents
      if(index(ic,'OR').ne.0.or.index(ic,'AND').ne.0.or.index(ic,'&').ne.0) return
      if(index(ic,'AP').ne.0.or.index(ic,'LE').ne.0.or.index(ic,'GE').ne.0) return
      if(index(ic,'LT').ne.0.or.index(ic,'GT').ne.0.or.index(ic,'NOT').ne.0) return
      if(index(ic,'+X').ne.0.or.index(ic,'+Y').ne.0.or.index(ic,'+Z').ne.0) return
      if(index(ic,'>').ne.0.or.index(ic,'<').ne.0) return
      if(index(ic,'I').ne.0.or.index(ic,':').ne.0.or.index(ic,'NAT').ne.0) return
      if(index(ic,'J').ne.0.or.index(ic,'P').ne.0.or.index(ic,'HI').ne.0) return
      if(index(ic,'TO').ne.0) then
	  ito=index(ic,'TO')
	  ic(ito:ito+1)=' ,'
	  flagto=.true.
	  endif
      jpar=0
      l=1
      m=0
      n=0
!Correction for 3/2(5/2) case. we insert a coma to get 3/2,(5/2) Oct. 2001 by T. Belgya
	  do i=2,17
	  flag=(ic(i:i).eq.'('.or.ic(i:i).eq.'[')
	  if(flag) then
	  flag=flag.and.(ic(i-1:i-1).ge.'0'.and.ic(i-1:i-1).le.'9')
	  flag=flag.and.(ic(i+1:i+1).ge.'0'.and.ic(i+1:i+1).le.'9')
	  endif
	  if(flag) then
	  do ii=1,i-2
	  ic(ii:ii)=ic(ii+1:ii+1)
	  enddo
	  ic(i-1:i-1)=','
	  endif
	  enddo
      do 10 i=1,18
      if(ic(i:i).eq.' ') go to 10
      if(ic(i:i).eq.'/') go to 10
      if(ic(i:i).eq.'2'.and.ic(i-1:i-1).eq.'/') then
	  odd=.true.
	  go to 10
	  endif

      if(ic(i:i).ne.'('.and.ic(i:i).ne.'[') go to 11
	  jpar=1
      m=l
      go to 10
11    continue
      if(ic(i:i).ne.')'.and.ic(i:i).ne.']') go to 12
      jpar=0
      n=l
      go to 10
12    continue
      if(ic(i:i).ne.',') go to 13
!c  avoid array overflow
	if(l.eq.5) go to 10
      l=l+1
      if(mjpi(l)(1:1).eq.' ') mjpi(l)=',  '
      go to 10
13    continue
      if(ic(i:i).ne.'-'.and.ic(i:i).ne.'+') go to 14
      pi(l)=ic(i:i)
      if(mjpi(l)(3:3).eq.'J'.and.jpar.eq.1) mjpi(l)(3:3)='B'
      if(mjpi(l)(3:3).eq.' '.and.jpar.eq.1) mjpi(l)(2:3)='(P'
      if(m.eq.0) go to 10
      do 20 kk=m,n
20    pi(kk)=ic(i:i)
      m=0
      n=0
      go to 10
14    continue
      if(ic(i:i).ne.'O'.and.ic(i:i).ne.'A') go to 15
      if(ic(i:i).ne.'O') go to 16
      ic(i:i+1)=' ,'
      mjpi(l)(1:3)='O  '
      go to 10
16    continue
      ic(i:i+2)=' , '
      mjpi(l)(1:3)='A  '
      go to 10
15    continue
      read(ic(i:i),100,err=99) k
100   format(i1)
      if(j(l).ge.0) j(l)=10*j(l)+k
      if(j(l).lt.0) j(l)=k
      if(jpar.eq.1) mjpi(l)(2:3)='(J'
      go to 10
99    continue
      WRITE(4,'(/'' ** Illegal JPI field: '',A)') IC
      ierr=ic(i:i)
10    continue
	  if(flagto) then
	  kk=0
	  jstep=1
	  j1=j(1)
	  j2=j(2)
	  p1=pi(1)
	  p2=pi(2)
	  if(.not.odd.and.j2-j1.gt.4) j2=j1+4
	  if(odd.and.j2-j1.gt.8) j2=j1+8
	  if(odd) jstep=2
	  do k=j1,j2,jstep
	  kk=kk+1
	  j(kk)=k
	  if(p1.eq.p2.and.(p1.ne.' '.and.p2.ne.' ')) then
	  pi(kk)=p1
	  else
	  pi(kk)=' '
	  endif

	  enddo
	  l=kk
	  endif
      return
      end

      Subroutine fend(ic1,ic2,x,mx,nx)
!c  Decode ENSDF data into numeric data
!c  String NX returns non-numerical uncertainty

      implicit DOUBLE PRECISION (a-h,o-z)
      Dimension x(3)
      Character*(*) ic1,ic2,mx,nx
      character*20 jc,kc,jbl

      k=len(ic1)
      kk=len(ic2)
	  mx='   '
      nx='  '

      do i=1,20
         jbl(i:i)=' '
         kc(i:i)=' '
         jc(i:i)=' '
      enddo

      x(1)=0.0
      x(2)=0.0
      x(3)=0.0
!CBlank value - return
      IF(IC1(1:K).EQ.JBL(1:K)) RETURN

!c Check for nonnumeric quantities
      jc(1:k)=ic1
1      n=index(jc,'+')

!CPrevious character is E (exponent)
!c$$   if(n.eq.0.or.jc(n-1:n-1).eq.'E') go to 10
      if(n.le.1.or.jc(n-1:n-1).eq.'E') go to 10

!CFollowing character is a number
      if(numb(jc(n+1:n+1)).ne.0) go to 15
      mx(1:2)=jc(n:n+1)
      jc(n:n+1)='  '
      if(n+2.gt.k.or.jc(n+2:n+2).eq.' ') go to 100
      mx(3:3)=jc(n+2:n+2)
      jc(n+2:n+2)=' '
      go to 100
15    continue
      mx(2:3)=jc(n-1:n)
      jc(n-1:n)='  '
      if(n-2.le.0.or.jc(n-2:n-2).eq.' ') go to 100
      mx(1:1)=jc(n-2:n-2)
      jc(n-2:n-2)=' '
      go to 100
10    continue
      n=index(jc,'?')
      if(n.eq.0) go to 20
      jc(n:n)=' '
      mx='  ?'
      go to 100
20    continue
      n=index(jc,'(')
      if(n.eq.0) go to 30
      mx=' ()'
      jc(n:n)=' '
      n=index(jc,')')
      if(n.gt.0) jc(n:n)=' '
      go to 100
30    continue
      n=index(jc,'X')
      if(n.eq.0) go to 40
      jc(n:n+2)='0+X'
      go to 1
40    continue
      n=index(jc,'Y')
      if(n.eq.0) go to 45
      jc(n:n+2)='0+Y'
      go to 1
45    continue
      n=index(jc,'SN')+index(jc,'SP')
      if(n.eq.0) go to 100
      jc(n:n+1)='  '
      mx(2:3)=jc(n:n+1)
      go to 1
100   continue
      read(jc(1:k),1000,err=101) x(1)
1000  format(bn,F20.0)
      go to 150
101   continue
      iii=1
      do 110 i=1,k
      if(numb(jc(i:i)).eq.1.or.jc(i:i).eq.'.') go to 110
      jc(i:i)=' '
      if(iii.lt.3) mx(iii:iii)=jc(i:i)
      iii=iii+1
110   continue
150   continue

!c  Check uncertainties for alphanumerics
      jj=0
      if(ic2(1:kk).eq.jbl(1:kk)) go to 99
      do 50 i=1,kk
         if(ic2(i:i).eq.'-'.or.ic2(i:i).eq.'+') go to 50
         if(ic2(i:i).ne.' '.and.numb(ic2(i:i)).eq.0) go to 200
         if(numb(ic2(i:i)).eq.1) jj=i
50    continue

      if(jj.eq.0) go to 99
      go to 500

200   continue
      nx=ic2(i:i+1)
      go to 99

500   continue

!c   Process uncertainties
      mm=k
      nq=0
      n=0
      n=index(jc,'E')
      n=n+index(jc,'e')
      if(n.eq.0) go to 505
      mm=n-1
      kc(n:k)=jc(n:k)
505   continue
      m1=2
506   continue
      do 510 i=1,mm
      n=mm-i+1
      if(jc(n:n).eq.' ') kc(n:n)=' '
      if(jc(n:n).eq.'.') kc(n:n)='.'
      if(jc(n:n).eq.' '.or.jc(n:n).eq.'.') go to 510
      kc(n:n)='0'
      if(nq.eq.1) go to 510
512   continue
      if(jj.le.0) go to 513
      if(ic2(jj:jj).ne.' ') go to 511
      jj=jj-1
      if(jj.gt.0) go to 512
      go to 513
511   continue
      kc(n:n)=ic2(jj:jj)
      jj=jj-1
513   continue
      if(jj.le.0.or.ic2(jj:jj).eq.'-'.or.ic2(jj:jj).eq.'+') nq=1
510   continue
      read(kc(1:k),1000) x(m1)
      if(m1.eq.3.or.x(3).gt.1.0D-20) go to 99
      if(ic2(jj:jj).ne.'-'.and.ic2(jj:jj).ne.'+') go to 98
      if(ic2(jj:jj).eq.'+') go to 515
      x(3)=x(2)
      x(2)=0.0
      ml=2
      go to 516
515   continue
      m1=3
516   continue
      kc=jbl
      jj=jj-1
      nq=0
      go to 500
98    continue
      x(3)=x(2)
99    continue
      return
      end

      SUBROUTINE SEC2UN(X,UNIT,lim)
!CScales half-life/errors (X) so that T1/2 is given in an
!Cappropriate unit.
!C
      implicit none
      CHARACTER*(*) lim,UNIT,TUNITS(17)*3
      double precision TFACT(17)
      double precision X(3),T,TT
      integer i,iunit

!C
!C1 YEAR IS 365.256 DAYS
!C
!CT1/2(S)=LN(2)*HBAR/WIDTH(EV) WHERE
!CLN(2)*HBAR=C=4.5624*10**-16 EV*S --> 1/C=2.1918*10**15
!C

      DATA TUNITS /'MEV','KEV',' EV',' AS',' FS',' PS',' NS', &
        ' US',' MS',' S ',' M ',' H ',' D ',' Y ',' KY',' MY',' GY'/

      DATA TFACT /0.45624D-21,0.45624D-18,0.45624D-15,1.D-18,1.D-15, &
      1.D-12,1.D-9,1.D-6,1.D-3,1.D+0,60.D+0,3600.D+0, &
      8.64D+4,3.155812D+7,3.155812D+10,3.155812D+13,3.155812D+16/


!CDetermine the appropriate unit
      if(x(1).lt.0.1D-18) then
         unit=' S  '
         return
      elseif(x(1).le.1.D-16) then
!C   AS
         iunit=4
         goto 10
      elseif(x(1).le.1.D-13) then
!C   FS
         iunit=5
         goto 10
      elseif(x(1).le.1.D-10) then
!C   PS
         iunit=6
         goto 10
      elseif(x(1).le.1.D-7) then
!C   NS
         iunit=7
         goto 10
      elseif(x(1).le.1.D-4) then
!C   US
         iunit=8
         goto 10
      elseif(x(1).le.1.D-1) then
!C   MS
         iunit=9
         goto 10
      elseif(x(1).lt.60.) then
!C   S
         iunit=10
         goto 10
      elseif(x(1).lt.3600.) then
!C   M
         iunit=11
         goto 10
      elseif(x(1).lt.86400.) then
!C   H
         iunit=12
         goto 10
      elseif(x(1).lt.3.155812D+7) then
!C   D
         iunit=13
         goto 10
      elseif(x(1).lt.3.155812D+10) then
!C   Y
         iunit=14
         goto 10
      elseif(x(1).lt.3.155812D+13) then
!C   KY
         iunit=15
         goto 10
      elseif(x(1).lt.3.155812D+16) then
!C   MY
         iunit=16
         goto 10
      elseif(x(1).lt.3.155812D+19) then
!C   GY
         iunit=17
         goto 10
      endif

      unit=' S  '
      return

10    continue
!CConvert half-life and errors from seconds to UNIT
      unit=tunits(iunit)
      x(1)=x(1)/tfact(iunit)
      x(2)=x(2)/tfact(iunit)
      x(3)=x(3)/tfact(iunit)

      return

30    entry UN2SEC(X,UNIT,LIM)

      if(x(1).le.0) return

      do i=4,17
         if(unit.eq.tunits(i)) then
!C      Convert half-life and errors from UNIT to seconds
            x(1)=x(1)*tfact(i)
            x(2)=x(2)*tfact(i)
            x(3)=x(3)*tfact(i)
            unit=' S '
            goto 90
          endif
      enddo

      do i=1,3
         if(unit.eq.tunits(i)) then
!C      Convert width and errors from UNIT to seconds
            t=x(2)/x(1)
            tt=x(3)/x(1)
            x(1)=tfact(i)/x(1)
            x(2)=x(1)*tt
            x(3)=x(1)*t
            unit=' S  '
!C      Reverse limits
            if(lim.ne.'  ') call lbsup(lim)
            if(lim(1:1).eq.'L') then
               lim(1:1)='G'
            elseif(lim(1:1).eq.'G') then
               lim(1:1)='L'
            endif
            goto 90
          endif
      enddo


90    continue
      return
      end



      SUBROUTINE ZNAME(EL,Z,IOPT)

!C  IOPT=0: Determine Z from element symbol EL.
!C  IOPT=1: Determine Z from element symbol EL, replace EL.
!C  IOPT=2: Determine element symbol from Z (Z-100 for Z>109).
!C  IOPT=3: Determine element symbol from Z (Z-100 for Z>103).

      IMPLICIT NONE
      CHARACTER*2 EL,NZ(110),LNZ(110)
      INTEGER Z,IOPT,I,NUMB

      DATA NZ/'H ','HE','LI','BE','B ','C ','N ','O ','F ', &
     'NE','NA','MG','AL','SI','P ','S ','CL','AR','K ','CA','SC','TI', &
     'V ','CR','MN','FE','CO','NI','CU','ZN','GA','GE','AS','SE','BR', &
     'KR','RB','SR','Y ','ZR','NB','MO','TC','RU','RH','PD','AG','CD', &
     'IN','SN','SB','TE','I ','XE','CS','BA','LA','CE','PR','ND','PM', &
     'SM','EU','GD','TB','DY','HO','ER','TM','YB','LU','HF','TA','W ', &
     'RE','OS','IR','PT','AU','HG','TL','PB','BI','PO','AT','RN','FR', &
     'RA','AC','TH','PA','U ','NP','PU','AM','CM','BK','CF','ES','FM', &
     'MD','NO','LR','RF','HA','SG','NS','HS','MT','10'/

      DATA LNZ/'H ','He','Li','Be','B ','C ','N ','O ','F ', &
     'Ne','Na','Mg','Al','Si','P ','S ','Cl','Ar','K ','Ca','Sc','Ti', &
     'V ','Cr','Mn','Fe','Co','Ni','Cu','Zn','Ga','Ge','As','Se','Br', &
     'Kr','Rb','Sr','Y ','Zr','Nb','Mo','Tc','Ru','Rh','Pd','Ag','Cd', &
     'In','Sn','Sb','Te','I ','Xe','Cs','Ba','La','Ce','Pr','Nd','Pm', &
     'Sm','Eu','Gd','Tb','Dy','Ho','Er','Tm','Yb','Lu','Hf','Ta','W ', &
     'Re','Os','Ir','Pt','Au','Hg','Tl','Pb','Bi','Po','At','Rn','Fr', &
     'Ra','Ac','Th','Pa','U ','Np','Pu','Am','Cm','Bk','Cf','Es','Fm', &
     'Md','No','Lr','Rf','Ha','Sg','Ns','Hs','Mt','10'/

!c  Get Z from EL
      IF(IOPT.LE.1) THEN

!c  Make sure element symbol is left justified
         IF(EL(1:1).EQ.' ') THEN
            EL(1:1)=EL(2:2)
            EL(2:2)=' '
         ENDIF

!c  Element symbol is number: Z = symbol + 100
         IF(numb(el(1:1)).eq.1.and.numb(el(2:2)).eq.1) THEN
            read(EL(1:2),'(I2)') Z
            Z=Z+100
         ELSE
            Z=0
            DO I=1,109
               IF(NZ(I).EQ.EL) Z=I
               IF(LNZ(I).EQ.EL) Z=I
            ENDDO
         ENDIF

!c  For IOPT=1, replace EL with proper element symbol
         IF(IOPT.EQ.1) THEN
            IF(Z.GT.0.and.Z.LT.110) THEN
               EL=LNZ(Z)
            ELSEIF(Z.GE.110.and.Z.LT.200) THEN
               write(EL(1:2),'(I2.2)') Z-100
            ELSE
               EL='  '
            ENDIF
         ENDIF

      ELSEIF(IOPT.EQ.2) THEN
         IF(Z.LE.0.OR.Z.GE.200) THEN
            EL='  '
         ELSEIF(Z.LE.109) THEN
            EL=LNZ(Z)
         ELSEIF(Z.GT.109) THEN
            WRITE(EL,'(I2.2)') Z-100
         ENDIF

      ELSEIF(IOPT.EQ.3) THEN
         IF(Z.LE.0.OR.Z.GE.200) THEN
            EL='  '
         ELSEIF(Z.LE.103) THEN
            EL=NZ(Z)
         ELSEIF(Z.GT.103) THEN
            WRITE(EL,'(I2.2)') Z-100
         ENDIF
      ENDIF

      RETURN
      END

      function numb(c)
!c  This function returns 1 if c is an integer between 0 and 9.

      IMPLICIT NONE
      character*1 C
      INTEGER N,NUMB

      n=ichar(c)
      numb=0
      if(n.gt.47.and.n.lt.58) numb=1
      return
      end

      subroutine stopit(str1,str2,str3,str4,number)
!c  Flags fatal errors and stops execution

      implicit none
      character*(*) str1,str2,str3,str4
      integer number

      write(4,*)
      write(4,'(A)')' ***** Fatal error *****'
      write(4,'(A)')' Dataset/Last card/message/number:'
      if(str1.ne.' ') write(4,'(A)') str1
      if(str2.ne.' ') write(4,'(A)') str2
      if(str3.ne.' ') write(4,'(A)') str3
      if(str4.ne.' ') write(4,'(A)') str4
      if(number.ne.0) write(4,*) number

      write(*,*)
      write(*,'(A)')' ***** Fatal error *****'
      write(*,'(A)')' Dataset/Last card/message/number:'
      if(str1.ne.' ') write(*,'(1X,A)') str1
      if(str2.ne.' ') write(*,'(1X,A)') str2
      if(str3.ne.' ') write(*,'(1X,A)') str3
      if(str4.ne.' ') write(*,'(1X,A)') str4
      if(number.ne.0) write(*,*) number

      stop
      end

      subroutine cardout(out,ocard)
!c  Write ocard witout trailing blanks to file out

      implicit none
      integer out,lenstr
      character*(*) ocard

      call tbsup(ocard)
      write(out,'(A)') ocard(1:lenstr(ocard))

      return
      end
      subroutine chgam(ein,dein,efi,defi,eg,deg,A,elo,delo,ehi,dehi,logon)
!C
!c  Make sure that gamma fits level energy difference. For recoil
!c  correction feed mass number as parameter A, for no recoil correction
!c  feed 999999. Errors are logged to unit 4.
!C
      implicit none
      double precision ein,efi,eg,dein,defi,deg,elo,delo,ehi,dehi
      real dl,ddl,err,uncgam,rec,diff,aa
      integer a,nsig,logon
      aa=a
!CIf A unknown, take a reasonable value
      if(a.le.1) aa=100.

!CSkip if level uncertainty not given except for G.S.
      if(dein.lt.0.000001) return
      if(defi.lt.0.000001.and.efi.gt.0.000001) return

!CUse 1 keV as default Eg uncertainty
      uncgam=deg
      if(deg.lt.0.000001) uncgam=1.0

      dl=ein-efi
      ddl=sqrt(dein*dein+defi*defi)
      err=sqrt(ddl*ddl+uncgam*uncgam)

!CCalculate recoil correction
      rec=eg*eg/(1863003.2*aa)

      diff=abs(dl-eg-rec)
      NSIG=NINT(diff/err)
      if(diff.gt.5*err) then
         if(NSIG.lt.10.or.diff.lt.0.5) THEN
            if(logon.eq.0) goto 200  
            write(4,*)
            write(4,102) '*W*',NSIG
102         format(1X,A,' Gamma does not fit between levels, deviation=', &
                  I2,' sigma')
         else
            write(4,*)
            write(4,104) '*E*',NSIG
104         format(1X,A,' Gamma does not fit between levels, deviation=' &
                  ,I5,' sigma')
         endif
         write(4,100) ein,efi,eg,dl
100      format('     Levels:',F10.3,' and ',F10.3,'. Eg=',F10.3, &
               '   DeltaE=',F10.3)
         write(4,101) dein,defi,deg,ddl
101      format('            ',F10.3,'     ',F10.3,'.    ',F10.3, &
               '          ',F10.3)

         write(4,103) elo,delo,ehi,dehi
103      format('     Previous/next final level:'2F10.3,' and',2F10.3)
      endif
200   continue
      return
      end

      subroutine chmult(J1,Par1,JPiMod1,J2,Par2,JPiMod2, &
      ML,MLPar,MLMod,mult,Iodd,eg,el1,el2,d,dd,dd2, elo,delo,ehi,dehi, &
      logon)
!c  Make sure that gamma multipolarity is consistent with level spins
!C
!c  Arguments:
!c  Spin, parity, modstring of initial level
!c  Spin, parity, modstring of final level
!c  Multipolarities, parity, modstring of gamma
!c  ENSDF mult string, IODD 2 for odd A 1 for even A
!C
!c  Invalid multipolarity (MLMod='***') gives a warning (*W*)
!c  Both muliporarities inconsistent with JPi's: *E*
!c  One muliporarity inconsistent with JPi's: *W*

      implicit none
      logical OK1,OK2,OKpar
      double precision eg,el1,el2
      double precision elo,delo,ehi,dehi
      real d,dd,dd2
      integer J1,J2,ML(2),Iodd,jdiff,jsum,logon
      character*(*) Par1,Par2,JPiMod1,JPiMod2,MLMod,mult
      character*7 JPi1,JPi2,MLPar*1,Dpar*1

      call PutJPi(JPi2,J2,Par2,JPiMod2,Iodd)
      call PutJPi(JPi1,J1,Par1,JPiMod1,Iodd)

!CCheck for given mixing ratio and no second multipole
      if((d.ne.0.or.dd.ne.0).and.ML(2).lt.0) then
         write(4,102) '*E*',mult,d,dd,dd2,eg,el1,el2,JPi1,Jpi2
102      format(/1X,A,' Non-zero mixing ratio and no second multipole: ' &
              ,A/'     Mixing ratio ',F6.2,' +',F6.2,' -'F6.2/ &
                 '     Eg=',F10.3,' from level ',F10.3,' to level ', &
          F10.3,/'     Level spin/parities: '7X,A,' and ',8X,A)
      endif

      if (ML(1).lt.0.and.MLPar.eq.' ') return
      if (J1.lt.0.and.Par1.eq.' ') return
      if (J2.lt.0.and.Par2.eq.' ') return

!CInvalid multipolarity
      if (MLMod.eq.'***'.and.logon.gt.0) then
         write(4,100) '*W*',mult,eg,el1
100      format(/1X,A,' Invalid multipolarity: ',A/ &
                '     Eg=',F10.3,' from level ',F10.3)
         return
      endif

!CCalculate Spin difference and parity change
      jdiff=abs(J1-J2)
      jsum=(J1+J2)
!CNo spin check if any of the levels has unknown spin
      if(J1.eq.-1.or.J2.eq.-1) then
         jdiff=0
         jsum=199
      endif
      Dpar=' '
      if (Par1.eq.'+'.and.Par2.eq.'+') Dpar='+'
      if (Par1.eq.'-'.and.Par2.eq.'-') Dpar='+'
      if (Par1.eq.'-'.and.Par2.eq.'+') Dpar='-'
      if (Par1.eq.'+'.and.Par2.eq.'-') Dpar='-'

      OKpar=.true.
      OK1=.true.
      OK2=.true.
       
!CCompare with multipolarity

!CParity:
      if(Dpar.ne.' '.and.Dpar.ne.MLPar.and.MLPar.ne.' ') then
         OKpar=.false.
      endif

!CSpins:
      if(ML(1)*IODD.lt.jdiff.or.ML(1)*IODD.gt.jsum) then
         OK1=.false.
      endif
      if(ML(2).gt.0) then
         if(ML(2)*IODD.lt.jdiff.or.ML(2)*IODD.gt.jsum) then
            OK2=.false.
         endif
      endif

      if((.not.OKpar).or.(((.not.OK1).and.(.not.OK2)))) then
         write(4,101) '*E*',mult,eg,el1,el2,JPi1,JPi2
101      format(/1X,A,' Multipolarity inconsistent with spin/parities: ' &
              ,A/'     Eg=',F10.3,' from level ',F10.3,' to level ', &
          F10.3,/'     Level spin/parities: '7X,A,' and ',8X,A)
         write(4,103) elo,delo,ehi,dehi
103      format('     Previous/next final level:'2F10.3,' and',2F10.3)

      elseif(((.not.OK1).or.(.not.OK2)).and.(logon.ge.1)) then
         write(4,101) '*W*',mult,eg,el1,el2,JPi1,JPi2
         write(4,103) elo,delo,ehi,dehi
      endif

      return
      end

      SUBROUTINE MATCH(X,Y,I,J)
!c  Finds closest match of X to array Y(I)

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION Y(*)

      DS=1.0D10
      DO 10 M=1,I
         DX=DABS(X-Y(M))
         IF(DX-DS.GT.25.0) GO TO 20
         IF(DX.GT.DS) GO TO 10
         DS=DX
         J=M
10    CONTINUE
20    CONTINUE
      RETURN
      END
